--------------------------------------------------------------------------------------
-- Rematch_TSMPetValues - Extends Rematch's pet list by TSM market values
--------------------------------------------------------------------------------------
-- Rematch_TSMPetValues.lua
--
--[[

Rematch_TSMPetValues extends Rematch's pet list by TSM market values,
so you don't have to cage a pet to see its TSM item value.

You can browse through your whole pet collection and can choose which precious pet
is due to sell.

It uses TSM to gather the item value and displays the Market Value (DBMarket)
and the Global Market Value Average (DBGlobalMarketAvg).

No configuration needed, just install it and pet values will show up in Rematch,
but you can change the price sources in the configuration.

Rematch_PetValues requires Rematch and TradeSkillMaster v3 and TradeSkillMaster_AuctionDB.

Author: Caraxe
All Rights Reserved

Version 0.1.2 - 2017.11.19
------------------------------------------------------------------------------------]]

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")

addon.METADATA = {
	NAME = GetAddOnMetadata(..., "Title"),
	VERSION = GetAddOnMetadata(..., "Version")
}

local defaults = {
	profile = {
	},
	global = {
		valA = "DBMarket",
		valB = "DBGlobalMarketAvg",
	},
}

local function GetOptions(uiType, uiName, appName)
	if appName == addonName then
		local options = {
			type = "group",
			name = GetAddOnMetadata(addonName, "Title"),
			args = {
				desc1 = {
					type = "description",
					order = 0,
					name = GetAddOnMetadata(addonName, "Notes"),
				},
				desc1a = {
					type = "description",
					order = 0.05,
					name = "",
				},
				valAVal = {
					name = L["First Price Source"],
					desc = L["Choose a TSM price source"],
					type = "input",
					order = 0.1,
					width = "double",
					get = function()
						return addon.db.global.valA
						end,
					set = function(info, value)
						if value == "" then value = nil end
						addon.db.global.valA = value
					end,
				},
				desc2 = {
					type = "description",
					order = 0.2,
					name = L["Default is 'DBMarket', one of '/tsm sources'"],
					width = "double",
				},
				valBVal = {
					name = L["Second Price Source"],
					desc = L["Choose a TSM price source"],
					type = "input",
					order = 0.3,
					width = "double",
					get = function()
						return addon.db.global.valB
						end,
					set = function(info, value)
						if value == "" then value = nil end
						addon.db.global.valB = value
					end,
				},
				desc3 = {
					type = "description",
					order = 0.4,
					name = L["Default is 'DBGlobalMarketAvg', one of '/tsm sources'"],
					width = "double",
				},
			},
		}
		return options
	end
end


-- hooked Function is Rematch:FillPetListButton(button, petID, forLoadout)
local OldRFPLB = nil
local function NewRFPLB(obj, button, petID, forLoadout)
	OldRFPLB(obj, button, petID, forLoadout)

	local idType
	if Rematch.GetIDType then
		idType = Rematch.GetIDType(nil, petID)
	else
		return
	end

	local _, speciesID, tsmSpeciesID, customName, priceInfo, level, name, rarity
	level = 1 -- doesn't matter
	rarity = 3 -- doesn't matter

	if idType=="pet" then
		speciesID, customName, level, _, _, _, _, name = C_PetJournal.GetPetInfoByPetID(petID)
		tsmSpeciesID = speciesID
	elseif idType=="species" then
		name = C_PetJournal.GetPetInfoBySpeciesID(petID)
		tsmSpeciesID = petID
	else
		return
	end

	if Rematch.petInfo.Fetch then
		local petInfo = Rematch.petInfo:Fetch(petID)
		rarity = petInfo.rarity or rarity
	end

	if TSMAPI then
		if addon.db.global.valA then
			local itemValue = TSMAPI:GetItemValue("p:" .. tsmSpeciesID .. ":" .. level .. ":" .. tostring(rarity), addon.db.global.valA)
			if itemValue then
				itemValue = floor(itemValue / COPPER_PER_GOLD) * COPPER_PER_GOLD
				local itemValueString = TSMAPI:MoneyToString(itemValue, "OPT_TRIM")
				priceInfo = itemValueString
			else
				priceInfo = "-"
			end
		end

		if addon.db.global.valB then
			local itemValue = TSMAPI:GetItemValue("p:" .. tsmSpeciesID .. ":" .. level .. ":" .. tostring(rarity), addon.db.global.valB)
			if itemValue then
				itemValue = floor(itemValue / COPPER_PER_GOLD) * COPPER_PER_GOLD
				local itemValueString = TSMAPI:MoneyToString(itemValue, "OPT_TRIM")
				priceInfo = priceInfo .. " / " .. itemValueString
			else
				priceInfo = priceInfo .. " / -"
			end
		end
	end

	if (priceInfo == "-" or priceInfo == "- / -") then
		priceInfo = nil
	end

	if priceInfo then
		if customName then
			button.SubName:SetText(priceInfo .. ", " .. name)
		else
			button.SubName:SetText(priceInfo)
		end
		button.Name:SetHeight(21)
		button.SubName:Show()

		-- debug
		-- button.Name:SetText("p:" .. tsmSpeciesID .. ":" .. level .. ":" .. tostring(rarity))
	end
end

function addon:OnInitialize()
	addon.db = AceDB:New(addonName .. "DB", defaults, true)

	AceConfigRegistry:RegisterOptionsTable(addonName, GetOptions)
	local optionsFrame = AceConfigDialog:AddToBlizOptions(addonName, GetAddOnMetadata(addonName, "Title"))
	addon.optionsFrame = optionsFrame
end

function addon:OnEnable()
	if Rematch and Rematch.FillPetListButton and Rematch.GetIDType then
		OldRFPLB = Rematch.FillPetListButton
		Rematch.FillPetListButton = NewRFPLB
		print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", L[" hooked into Rematch"])
	else
		print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", L[" NOT hooked into Rematch"])
	end
end

-- EOF
